/* File Measure.c */


/* System include files */
extern "C" {
 #include <math.h>
 #include <stdlib.h>
 #include <graphics.h>
 #include <stdio.h>
 #include <string.h>
 #include <conio.h>
}

/* Custom include files */
// Note: in later stage no hardware dependent include files to be put in here!!!
// This file serves as a common measurement file!!!
#include "ad_tpex.h"
#include "ad_hs508.h"
#include "const.h"
#include "types.h"
#include "addmath.h"
#include "globvars.h"
#include "measure.h"

/* Software interrupts inits... */
#pragma intrinsic _enable
#pragma intrinsic _disable

#if defined __cplusplus
  #define __CPPARGS ...
#else
  #define __CPPARGS
#endif


// Local High-Level Measurement Functions
void InitializeInstrument (void);
void ResetInstrument(void);
void DeterminePrePostSamples (EQUIP_OVERALL_INFO* pOverallInfoLIVE, long* pNumPreSamples, long* pfNumPostSamples);
void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies);
void StartMeasuringHS508 (MEASDEP* pMeasDependencies, EQUIP_OVERALL_INFO* pLIVEInfo);

/* Prototypes for interrupt functions */
void interrupt (*oldtimer)(__CPPARGS);
void interrupt newtimer(__CPPARGS);

/* Global variables */
long lTimerCount;


/* NAME:
 *   void InitializeInstrument (void)
 * DESCRIPTION:
 *   This routine initializes the timer interrupt,
 *     checks the presence of the HS508,
 *     initializes the DA converters of the card
 *     calls the calibration routines
 *   If all is successfull, the program continues, otherwise the program is
 *   terminated with an error code 1.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void InitializeInstrument (void)
{

 int nChannelIndex, nStand;
 char TextBuffer[80];

/*
 * Replace existing timer-interrupt
 */
 oldtimer = getvect( 0x1c );
 setvect( 0x1c, newtimer);

/*
 * check whether the card is present
 */
 if (!IsHardwarePresent())
 {
 /*
  * No it is not present
  */
  cleardevice();
  sprintf (TextBuffer, "Could not find the HS508");
  outtextxy (50, 220, TextBuffer);
  sprintf (TextBuffer, "Please connect HS508 to your PC prior to executing this program...");
  outtextxy (50, 250, TextBuffer);
  do
  {
  } while ( !kbhit() );
  closegraph();
  ResetInstrument();
  exit (1);
 }

/*
 * clear the waveform memory
 */
 CalibrateInstrument();
 ClearSRAM ();

/*
 * initialize the AD converters
 */
 bCanMeasure = TRUE;

 cleardevice();

}//InitializeInstrument



/* NAME:
 *   void interrupt newtimer(__CPPARGS)
 * DESCRIPTION:
 *   This is the new timer interrupt routine
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void interrupt newtimer(__CPPARGS)
{
  oldtimer();
  _disable();
  lTimerCount++;
  _enable ();
} //newtimer(__CPPARGS)



/* NAME:
 *   void ResetInstrument
 * DESCRIPTION:
 *   This routine resets the original timer interrupt
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void ResetInstrument (void)
{
 // Reset to old timer-interrupt
 setvect(0x1c, oldtimer);
}//ResetInstrument



/* NAME:
 *   void StartMeasuringHS508( MEASDEP* pMeasDependencies,
 *                             EQUIP_OVERALL_INFO* pLIVEInfo
 *                           )
 * DESCRIPTION:
 *   This routine performs one measurement with the HS508 and displays
  *  the measured data like an oscilloscope.
 * INPUT PARAMETERS:
 *   pMeasDependencies  settings of the instrument (v/div, s/div etc.)
 *   pLIVEInfo          measurement mode (freeze or live)
 * OUTPUT PARAMETERS:
 *   -
 */
void StartMeasuringHS508( MEASDEP*            pMeasDependencies,
                          EQUIP_OVERALL_INFO* pLIVEInfo
                        )
{
  HS508_MEASDEP HS508_OverallInfo;
  long int lSwitchSetting;

 /*
  * set the PGA, with the coupling and switch settings
  */
  switch ( pLIVEInfo->MeasurementMode ) {
    case CHAN1  : if (pMeasDependencies->SampleFreq > 25e+6)
                  {
                   /*
                    * 50MSamples/sec only available to CH1
                    */
                    lSwitchSetting = SwitchSetting[CHANNEL1][pMeasDependencies->Switch[CHANNEL1]]+
                                     SwitchSetting[CHANNEL2][pMeasDependencies->Switch[CHANNEL2]]-
                                     PGA1ON + PGA2ON;
                  }
                  else
                  {
                    lSwitchSetting = SwitchSetting[CHANNEL1][pMeasDependencies->Switch[CHANNEL1]]+
                                    SwitchSetting[CHANNEL2][pMeasDependencies->Switch[CHANNEL2]];
                  }
                  break;
    case CHAN2:   ;
    case ALT  :   ;
    case CHOP :   lSwitchSetting = SwitchSetting[CHANNEL1][pMeasDependencies->Switch[CHANNEL1]]+
                                   SwitchSetting[CHANNEL2][pMeasDependencies->Switch[CHANNEL2]];
                  break;
  }
  SetPGA( pMeasDependencies->ACDC[CHANNEL1],
          pMeasDependencies->ACDC[CHANNEL2],
          lSwitchSetting
        );
 /*
  * Set the offsets and the DC levels
  */
  SetDA(0,(pMeasDependencies->DCLevel[CHANNEL1]/SwitchValue[pMeasDependencies->Switch[CHANNEL1]])*
          DcOffsetGain + ValueRef[CHANNEL1][pMeasDependencies->Switch[CHANNEL1]]
       );
  SetDA(1,(pMeasDependencies->DCLevel[CHANNEL2]/SwitchValue[pMeasDependencies->Switch[CHANNEL2]])*
          DcOffsetGain + ValueRef[CHANNEL2][pMeasDependencies->Switch[CHANNEL2]]
       );



  SetDA( 2, (-512 + ValueDA[CHANNEL1][pMeasDependencies->Switch[CHANNEL1]]) *
            (2.5 / 1024)
       );
  SetDA( 3, (-512 + ValueDA[CHANNEL2][pMeasDependencies->Switch[CHANNEL2]]) *
            (2.5 / 1024)
       );
 /*
  * prepare some more settings
  */
  HS508_OverallInfo.SampleFreq      = pMeasDependencies->SampleFreq;
  HS508_OverallInfo.NumPostSamples  = pMeasDependencies->NumPostSamples;
  HS508_OverallInfo.NumPreSamples   = pMeasDependencies->NumPreSamples;
  HS508_OverallInfo.TrigHysteresis  = pMeasDependencies->TrigHysteresis;
  HS508_OverallInfo.TrigLevel       = pMeasDependencies->TrigLevel;
  HS508_OverallInfo.TrigSlope       = pMeasDependencies->TrigSlope;
  HS508_OverallInfo.TrigSource      = pMeasDependencies->TrigSource;
  HS508_OverallInfo.MeasurementMode = pLIVEInfo->MeasurementMode;

 /*
  * perform one measurement, with the prepared settings
  */
  MeasureHS508( &HS508_OverallInfo );
  _disable();
  lTimerCount = 0;
  _enable();
  do
  {
   /*
    * the timer interrupt service routine increase timer counter until
    * End Of Conversion;
    * if TimeOut value is reached, set the triggerbit
    */
    if (lTimerCount >= pMeasDependencies->TimeOut)
    {
      SetTrigBit();
    }
  }
  while ( !EOC() );

  if ( EOC() )
  {
   /*
    * if conversion is ready, copy waveforms to PC RAM,
    * extract the samples and display them
    */
    RetrieveDataFromBoard (&HS508_OverallInfo);
    ExtractSamples (pLIVEInfo);
    DisplayMeasuredSignals (pLIVEInfo);
  }
}//StartMeasuringHS508



/* NAME:
 *   void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies)
 * DESCRIPTION:
 *   This routine initializes the instrument settings
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pMeasDependencies   this structure will contain the instrument setup
 */
void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies)
{
 pMeasDependencies->Switch[CHANNEL1]   = PGA2VOLT;
 pMeasDependencies->Switch[CHANNEL2]   = PGA2VOLT;
 pMeasDependencies->ACDC[CHANNEL1]     = COUPLEDC;
 pMeasDependencies->ACDC[CHANNEL2]     = COUPLEDC;
 pMeasDependencies->DCLevel[CHANNEL1]  = 0.0;
 pMeasDependencies->DCLevel[CHANNEL2]  = 0.0;
 pMeasDependencies->TrigSource         = CH1;
 pMeasDependencies->TrigLevel          = 128;       // 128 = midscreen
 pMeasDependencies->TrigHysteresis     = 3;
 pMeasDependencies->TrigSlope          = NEG;
 pMeasDependencies->NumPreSamples      = 100;
 pMeasDependencies->NumPostSamples     = 300;
 pMeasDependencies->SampleFreq         = 1e4;
 pMeasDependencies->TimeOut            = 0;  // 5 * (1/18.2) sec

} //InitializeMeasurementDependencies
