/* File Measure.c */


/* System include files */
extern "C" {
 #include <math.h>
 #include <stdlib.h>
 #include <graphics.h>
 #include <stdio.h>
 #include <string.h>
}

/* Custom include files */
// Note: in later stage no hardware dependent include files to be put in here!!!
// This file serves as a common measurement file!!!
#include "ad_tpex.h"
#include "ad_tp112.h"
#include "const.h"
#include "types.h"
#include "addmath.h"
#include "globvars.h"
#include "measure.h"

/* Software interrupts inits... */
#pragma intrinsic _enable
#pragma intrinsic _disable

#if defined __cplusplus
  #define __CPPARGS ...
#else
  #define __CPPARGS
#endif


// Local High-Level Measurement Functions
void InitializeTP112 (void);
void ResetTP112(void);
void DeterminePrePostSamples (EQUIP_OVERALL_INFO* pOverallInfoLIVE, long* pNumPreSamples, long* pfNumPostSamples);
BOOL IsHardwarePresent (void);
void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies);
void StartMeasuringTP112 (MEASDEP* pMeasDependencies, EQUIP_OVERALL_INFO* pLIVEInfo);

/* Prototypes for interrupt functions */
void interrupt (*oldtimer)(__CPPARGS);
void interrupt newtimer(__CPPARGS);

/* Global variables */
long lTimerCount;



/*
 * Measure.c function definitions...
 *
 * NAME:
 *   BOOL IsHardwarePresent (void)
 * DESCRIPTION:
 *   This routine checks whether the hardware of the TP112 is present
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   returnvalue  TRUE   hardware is     present
 *                FALSE  hardware is not present
 */
BOOL IsHardwarePresent (void)
{
  int nCount, nValue = 0;

  for (nCount =0; nCount < 10; nCount++)
  {
    nValue+=inportb (Ad_PortAddress + 3);
  }
  if (nValue == 0)
  {
    return (TRUE);
  }
  else /* nValue <> 0 */
  {
    return (FALSE);
  }
}  //IsHardwarePresent


/* NAME:
 *   void InitializeTP112 (void)
 * DESCRIPTION:
 *   This routine initializes the timer interrupt,
 *     checks the presence of the TP112 card,
 *     initializes the DA converters of the card
 *     calls the calibration routines
 *   If all is successfull, the program continues, otherwise the program is
 *   terminated with an error code 1.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void InitializeTP112 (void)
{

 int nChannelIndex, nStand;
 char TextBuffer[80];

/*
 * Replace existing timer-interrupt
 */
 oldtimer = getvect( 0x1c );
 setvect( 0x1c, newtimer);

/*
 * check whether the card is present
 */
 if (!IsHardwarePresent)
 {
 /*
  * No it is not present
  */
  cleardevice();
  sprintf (TextBuffer, "Could not find TP112-Board in your PC!!!");
  outtextxy (50, 220, TextBuffer);
  sprintf (TextBuffer, "Please connect TP112-Board in your PC prior to executing this program...");
  outtextxy (50, 250, TextBuffer);
  closegraph();
  ResetTP112();
  exit (1);
 }

/*
 * clear the waveform memory at the card
 */
 ClearSRAM ();

/*
 * initialize the AD converters
 */
 bCanMeasure = TRUE;

 cleardevice();

}//InitializeTP112



/* NAME:
 *   void interrupt newtimer(__CPPARGS)
 * DESCRIPTION:
 *   This is the new timer interrupt routine
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void interrupt newtimer(__CPPARGS)
{
  oldtimer();
  _disable();
  lTimerCount++;
  _enable ();
} //newtimer(__CPPARGS)



/* NAME:
 *   void ResetTP112 (void)
 * DESCRIPTION:
 *   This routine resets the TP112 after use and resets the
 *   original timer interrupt
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void ResetTP112 (void)
{
 outportb (int(Ad_PortAddress + 4), (unsigned char)3);
 outportb (int(Ad_PortAddress + 5), (unsigned char)0);
 // Reset to old timer-interrupt
 setvect(0x1c, oldtimer);
}//ResetTP112



/* NAME:
 *   void StartMeasuringTP112( MEASDEP* pMeasDependencies,
 *                             EQUIP_OVERALL_INFO* pLIVEInfo
 *                           )
 * DESCRIPTION:
 *   This routine performs one measurement with the TP112 card and displays
  *  the measured data like an oscilloscope.
 * INPUT PARAMETERS:
 *   pMeasDependencies  settings of the instrument (v/div, s/div etc.)
 *   pLIVEInfo          measurement mode (freeze or live)
 * OUTPUT PARAMETERS:
 *   -
 */
void StartMeasuringTP112( MEASDEP*            pMeasDependencies,
                          EQUIP_OVERALL_INFO* pLIVEInfo
                        )
{
 TP112_MEASDEP TP112_OverallInfo;

/*
 * set the PGA, with the coupling and switch settings
 */
 SetPGA( CHANNEL1,
         pMeasDependencies->ACDC[CHANNEL1],
         pMeasDependencies->Switch[CHANNEL1]
       );
 SetPGA( CHANNEL2,
         pMeasDependencies->ACDC[CHANNEL2],
         pMeasDependencies->Switch[CHANNEL2]
       );

/*
 * prepare some more settings
 */
 TP112_OverallInfo.SampleFreq      = pMeasDependencies->SampleFreq;
 TP112_OverallInfo.NumPostSamples  = pMeasDependencies->NumPostSamples;
 TP112_OverallInfo.NumPreSamples   = pMeasDependencies->NumPreSamples;
 TP112_OverallInfo.TrigHysteresis  = pMeasDependencies->TrigHysteresis;
 TP112_OverallInfo.TrigLevel       = pMeasDependencies->TrigLevel;
 TP112_OverallInfo.TrigSlope       = pMeasDependencies->TrigSlope;
 TP112_OverallInfo.TrigSource      = pMeasDependencies->TrigSource;
 TP112_OverallInfo.MeasurementMode = pLIVEInfo->MeasurementMode;

/*
 * perform one measurement, with the prepared settings
 */
 MeasureTP112( &TP112_OverallInfo );
 _disable();
 lTimerCount = 0;
 _enable();
 do
 {
  /*
   * the timer interrupt service routine increase timer counter until
   * End Of Conversion;
   * if TimeOut value is reached, set the triggerbit
   */
   if (lTimerCount >= pMeasDependencies->TimeOut)
   {
     SetTrigBit();
   }
 }
 while (!ReadEOCBit());

 if (ReadEOCBit())
 {
  /*
   * if conversion is ready, copy waveforms to PC RAM,
   * extract the samples and display them
   */
   RetrieveDataFromBoard (&TP112_OverallInfo);
   ExtractSamples (pLIVEInfo);
   DisplayMeasuredSignals (pLIVEInfo);
 }
}//StartMeasuringTP112



/* NAME:
 *   void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies)
 * DESCRIPTION:
 *   This routine initializes the instrument settings
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pMeasDependencies   this structure will contain the instrument setup
 */
void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies)
{
 pMeasDependencies->Switch[CHANNEL1]   = PGA5VOLT;
 pMeasDependencies->Switch[CHANNEL2]   = PGA5VOLT;
 pMeasDependencies->ACDC[CHANNEL1]     = COUPLEDC;
 pMeasDependencies->ACDC[CHANNEL2]     = COUPLEAC;
 pMeasDependencies->TrigSource         = CH1;
 pMeasDependencies->TrigLevel          = 128;       // 128 = midscreen
 pMeasDependencies->TrigHysteresis     = 3;
 pMeasDependencies->TrigSlope          = NEG;
 pMeasDependencies->NumPreSamples      = 100;
 pMeasDependencies->NumPostSamples     = 300;
 pMeasDependencies->SampleFreq         = 1e6;
 pMeasDependencies->TimeOut            = 5;   // 5 * (1/18.2) sec

} //InitializeMeasurementDependencies
