(*
 * This file contains a 'Public Domain' version of Handyscope software.
 * The software is written for a Handyscope with digital input.
 *)

program HS_PD;

uses crt;

const
  AddressLPT1   = $3BC;
  AddressLPT2   = $378;
  AddressLPT3   = $278;
  ADaddress     : integer = AddressLPT2;
  InputRange_05 = 1;
  InputRange_1  = 2;
  InputRange_2  = 3;
  InputRange_5  = 4;
  InputRange_10 = 5;
  InputRange_20 = 6;
  InputValue    : array[1..6] of real = ( 0.5, 1.0, 2.0, 5.0, 10.0, 20.0 );

var
  MeasurementArray : array[0..100] of byte; { array to contain measured values }
  Index            : word;                  { loop counter                     }
  InPutRangeCh1,
  InPutRangeCh2    : integer;

procedure SetSensitivity( InputRangeCh1, InputRangeCh2: byte );
(*
 * DESCRIPTION
 *   This routine sets the input sensitivity of the Handyscope.
 * INPUT
 *   bySetting1 : the sensitivity setting for ch1
 *   bySetting2 : the sensitivity setting for ch2
 * OUTPUT
 *   -
 *)
var byChSetting1,
    byChSetting2  : byte;
begin

 (*
  * convert sensitivity settings to bit patterns
  *)
  case InputRangeCh1 of
    InputRange_05 : byChSetting1 := 248 + 0;  {  0.5 volt }
    InputRange_1  : byChSetting1 := 248 + 4;  {  1   volt }
    InputRange_2  : byChSetting1 := 248 + 6;  {  2   volt }
    InputRange_5  : byChSetting1 := 248 + 1;  {  5   volt }
    InputRange_10 : byChSetting1 := 248 + 5;  { 10   volt }
    else            byChSetting1 := 248 + 7;  { 20   volt }
  end; { case }

  case InputRangeCh2 of
    InputRange_05 : byChSetting2 := 248 + 0;  {  0.5 volt }
    InputRange_1  : byChSetting2 := 248 + 4;  {  1   volt }
    InputRange_2  : byChSetting2 := 248 + 6;  {  2   volt }
    InputRange_5  : byChSetting2 := 248 + 1;  {  5   volt }
    InputRange_10 : byChSetting2 := 248 + 5;  { 10   volt }
    else            byChSetting2 := 248 + 7;  { 20   volt }
  end; { case }

 (*
  * Set the input sensitivity
  *)
  port[ADaddress  ] := byChSetting1;
  port[ADaddress  ] := byChSetting1;
  port[ADaddress  ] := byChSetting1;
  port[ADaddress+2] :=  0 + 4;       { slct is high                   }
  port[ADaddress+2] :=  0 + 4;       { slct is high                   }
  port[ADaddress+2] :=  0 + 4;       { slct is high                   }
  port[ADaddress+2] := 11 + 4;       { select channel 1 and clck high }
  port[ADaddress+2] := 11 + 4;       { select channel 1 and clck high }
  port[ADaddress+2] := 11 + 4;       { select channel 1 and clck high }
  port[ADaddress+2] :=  0 + 4;       { slct is high                   }
  port[ADaddress+2] :=  0 + 4;       { slct is high                   }
  port[ADaddress+2] :=  0 + 4;       { slct is high                   }

  port[ADaddress  ] := byChSetting2;
  port[ADaddress  ] := byChSetting2;
  port[ADaddress  ] := byChSetting2;
  port[ADaddress+2] :=  0;           { slct is high                   }
  port[ADaddress+2] :=  0;           { slct is high                   }
  port[ADaddress+2] :=  0;           { slct is high                   }
  port[ADaddress+2] := 11;           { select channel 2 and clck high }
  port[ADaddress+2] := 11;           { select channel 2 and clck high }
  port[ADaddress+2] := 11;           { select channel 2 and clck high }
  port[ADaddress+2] :=  0;           { slct is high                   }
  port[ADaddress+2] :=  0;           { slct is high                   }
  port[ADaddress+2] :=  0;           { slct is high                   }

  port[ADaddress  ] := 255;          { Apply total power supply       }
  port[ADaddress  ] := 255;          { Apply total power supply       }

end; { set sensitivity }


Function DigitInput : boolean;
(*
 * DESCRIPTION
 *   This routine reads the digital input of the Handyscope
 * INPUT
 *   -
 * OUTPUT
 *   returnvalue:       True  = digital input is high
 *                      False = digital input is low
 *)
begin

  port[ADaddress+2] := 2;            { ch1 off, ch2 on }
  port[ADaddress+2] := 3;            { ch1 off, ch2 on }
  port[ADaddress+2] := 2;            { ch1 off, ch2 on }

  if   ((port[ADaddress+1] shr 3) and 1) = 1
  then DigitInput := True
  else DigitInput := False;

end;


procedure MeasureHandyscope( nMeasureCount, nMeasureDelay : integer );
(*
 * DESCRIPTION
 *   This routine measures a requested number of values from the Handyscope,
 *   using a requested delay between the measurements.
 * The data is placed in the MeasurementArray
 * INPUT
 *   nMeasureCount      : indicates the number of values to measure
 *   nMeasureDelay      : indicates the time to wait between measurements
 * OUTPUT
 *
 *)
var wCounter  : word;  { counter used to create the measurement delay }
    wSampleNr : word;  { holds the samplenumber                       }
    D2_D6,             { help variable to hold the databits 2 to 6    }
    byDummy   : byte;  { used to perform a dummy read operation       }

begin
  wSampleNr := 0;
  repeat
   (*
    * read D7 .. D11
    *)
    port[ADaddress+2] := 7;  { ch1 off, ch2 on }

    wCounter := 0;
    repeat
      inc(wCounter);
    until wCounter >= nMeasureDelay;
    MeasurementArray[wSampleNr*4+2] := port[ADaddress+1];

   (*
    * read D2 .. D6
    *)
    port[ADaddress+2] := 5;  { ch1 off, ch2 on  ;  start conversion on ch2 }
    D2_D6 := port[ADaddress+1];

   (*
    * read nothing
    *)
    port[ADaddress+2] := 0;  { ch1 on, ch2 off }
    byDummy := port[ADaddress+1];

   (*
    * read D0 .. D1
    *)
    port[ADaddress+2] := 2;  { ch1 on, ch2 off }
    MeasurementArray[wSampleNr*4+3] := ( (port[ADaddress+1] and 96) shr 4 ) +
                                       (D2_D6 and 248);

   (*
    * read D7 .. D11
    *)
    port[ADaddress+2] := 3;  { ch1 on, ch2 off }
    wCounter := 0;
    repeat
      inc(wCounter);
    until wCounter >= nMeasureDelay;
    MeasurementArray[wSampleNr*4+0] := port[ADaddress+1];

   (*
    * read D2 .. D6
    *)
    port[ADaddress+2] := 1;  {ch1 on, ch2 off  ;   start conversion on ch1 }
    D2_D6 := port[ADaddress+1];

   (*
    * read nothing
    *)
    port[ADaddress+2] := 4;  { ch1 off, ch2 on }
    byDummy := port[ADaddress+1];

   (*
    * read D0 .. D1
    *)
    port[ADaddress+2] := 6;  { ch1 off, ch2 on }
    MeasurementArray[wSampleNr*4+1] := ( (port[ADaddress+1] and 96) shr 4) +
                                       (D2_D6 and 248);

    inc(wSampleNr);
  until wSampleNr >= nMeasureCount;

end; { MeasureHandyscope }


(*
 *
 * M A I N   P R O G R A M
 *
 *)
begin
  InPutRangeCh1 := InputRange_20;
  InPutRangeCh2 := InputRange_2;

  port[ADaddress] := 255; { Apply total power supply                 }
  delay( 10 );            { and wait a while after power is supplied }
  ClrScr;
  SetSensitivity(InPutRangeCh1,InPutRangeCh2);

  repeat
    GotoXY( 1, 1 );
   (*
    * nMeasureDelay must lie between 1 .. 65534, in this example the value
    * 100 is taken for nMeasureDelay
    * At fast computers nMeasureDelay usually has to be larger than 1
    *
    * Measure 15 values with the Handyscope, using 100 for delay
    *)
    MeasureHandyscope(15, 100 );
    for Index := 1 to 14 do
    begin
     (*
      * Display the measured values
      *
      * The first sample can not be used, therefore the loop starts at 1,
      * instead of 0
      *)
      writeln( Index:4,
      '  CH1 =',
      (((MeasurementArray[Index*4+0] and 248)*16+(MeasurementArray[Index*4+1] div 2))-2048)/2048*InputValue[InPutRangeCh1]:8:3,
      '  CH2 =',
      (((MeasurementArray[Index*4+2] and 248)*16+(MeasurementArray[Index*4+3] div 2))-2048)/2048*InputValue[InPutRangeCh2]:8:3
             );
    end;
    if   DigitInput
    then writeln('Digital input is high')
    else writeln('Digital input is low ');
  until keypressed;

end.
